export = pluginCreator;
/**
 * @type {import('postcss').PluginCreator<browserslist.Options>}
 * @param {browserslist.Options} options
 * @return {import('postcss').Plugin}
 */
declare function pluginCreator(options?: browserslist.Options): import('postcss').Plugin;
declare namespace pluginCreator {
    const postcss: true;
}
import browserslist = require("browserslist");
