module.exports={A:{A:{"2":"K D E F A B oC"},B:{"1":"0 9 Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB","2":"C L M G N O P","1028":"W X","1540":"Q H R S T U V"},C:{"1":"0 9 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC qC rC","2":"pC","164":"1 2 3 4 5 6 7 8 NC J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB sC tC","1540":"gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B"},D:{"1":"0 9 Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC","292":"1 2 3 4 5 6 7 8 J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B","1028":"W X","1540":"6B 7B 8B 9B AC BC CC DC EC FC Q H R S T U V"},E:{"1":"G 1C VC WC JC 2C KC XC YC ZC aC bC 3C LC cC dC eC fC gC 4C MC hC iC jC kC lC 5C","292":"J RB K D E F A B C uC TC vC wC xC yC UC HC","1540":"L M IC zC","3076":"0C"},F:{"1":"0 DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z","2":"F B C 6C 7C 8C 9C HC mC AD IC","292":"1 2 3 4 5 6 7 8 G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB","1028":"BC CC","1540":"vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC"},G:{"1":"UD VC WC JC VD KC XC YC ZC aC bC WD LC cC dC eC fC gC XD MC hC iC jC kC lC","292":"E TC BD nC CD DD ED FD GD HD ID JD KD LD MD","1540":"ND OD PD QD RD SD","3076":"TD"},H:{"2":"YD"},I:{"1":"I","292":"NC J ZD aD bD cD nC dD eD"},J:{"292":"D A"},K:{"1":"H","2":"A B C HC mC IC"},L:{"1":"I"},M:{"1":"GC"},N:{"2":"A B"},O:{"1":"JC"},P:{"1":"1 2 3 4 5 6 7 8 oD KC LC MC pD","292":"J fD gD hD iD jD","1540":"UC kD lD mD nD"},Q:{"1540":"qD"},R:{"1":"rD"},S:{"1":"tD","1540":"sD"}},B:5,C:"CSS Logical Properties",D:true};
