import * as Types from '../typebox';
export declare enum StructuralResult {
    Union = 0,
    True = 1,
    False = 2
}
/** Performs structural equivalence checks against TypeBox types. */
export declare namespace Structural {
    /** Structurally tests if the left schema extends the right. */
    function Check(left: Types.TSchema, right: Types.TSchema): StructuralResult;
}
