module.exports={A:{A:{"1":"A B","2":"K D E F oC"},B:{"1":"0 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB","2":"C L M G N O P"},C:{"1":"0 9 cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC qC rC","2":"pC NC sC tC","132":"1 2 3 4 5 6 7 8 J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB"},D:{"1":"0 7 8 9 TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC","2":"J","16":"2 3 4 5 6 RB K D E","132":"1 F A B C L M G N O P SB"},E:{"1":"C L M G HC IC zC 0C 1C VC WC JC 2C KC XC YC ZC aC bC 3C LC cC dC eC fC gC 4C MC hC iC jC kC lC 5C","2":"J RB uC TC vC","132":"K D E F A B wC xC yC UC"},F:{"1":"0 1 2 3 4 5 6 7 8 G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z","2":"F B C 6C 7C 8C 9C HC mC AD IC"},G:{"2":"DD ED","132":"E FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UD VC WC JC VD KC XC YC ZC aC bC WD LC cC dC eC fC gC XD MC hC iC jC kC lC","514":"TC BD nC CD"},H:{"2":"YD"},I:{"2":"ZD aD bD","260":"NC J cD nC","514":"I dD eD"},J:{"132":"A","260":"D"},K:{"2":"A B C HC mC IC","514":"H"},L:{"260":"I"},M:{"2":"GC"},N:{"514":"A","1028":"B"},O:{"2":"JC"},P:{"260":"1 2 3 4 5 6 7 8 J fD gD hD iD jD UC kD lD mD nD oD KC LC MC pD"},Q:{"260":"qD"},R:{"260":"rD"},S:{"1":"sD tD"}},B:1,C:"accept attribute for file input",D:true};
