import _readFromFixture from './readFromFixture';
export { _readFromFixture as readFromFixture };
//# sourceMappingURL=data:application/json;charset=utf-8;base64,eyJ2ZXJzaW9uIjozLCJzb3VyY2VzIjpbIi4uLy4uL3NyYy91dGlscy9pbmRleC5qcyJdLCJuYW1lcyI6WyJyZWFkRnJvbUZpeHR1cmUiXSwibWFwcGluZ3MiOiI2QkFBNEIsbUI7NkJBQXJCQSxlIiwiZmlsZSI6ImluZGV4LmpzIiwic291cmNlc0NvbnRlbnQiOlsiZXhwb3J0IHJlYWRGcm9tRml4dHVyZSBmcm9tICcuL3JlYWRGcm9tRml4dHVyZSc7XG4iXX0=