module.exports={A:{A:{"1":"K D E F A B oC"},B:{"1":"0 9 C L M G N O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC qC rC","2":"pC NC J RB K D E sC tC"},D:{"1":"0 1 2 3 4 5 6 7 8 9 J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC"},E:{"1":"K D E F A B C L M G wC xC yC UC HC IC zC 0C 1C VC WC JC 2C KC XC YC ZC aC bC 3C LC cC dC eC fC gC 4C MC hC iC jC kC lC 5C","16":"J RB uC TC vC"},F:{"1":"0 1 2 3 4 5 6 7 8 B C G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z 7C 8C 9C HC mC AD IC","16":"F 6C"},G:{"1":"E ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UD VC WC JC VD KC XC YC ZC aC bC WD LC cC dC eC fC gC XD MC hC iC jC kC lC","2":"TC BD","16":"nC CD DD"},H:{"2":"YD"},I:{"1":"I cD nC dD eD","2":"NC J ZD aD bD"},J:{"1":"A","2":"D"},K:{"1":"A B C H HC mC IC"},L:{"1":"I"},M:{"1":"GC"},N:{"1":"B","2":"A"},O:{"1":"JC"},P:{"1":"1 2 3 4 5 6 7 8 J fD gD hD iD jD UC kD lD mD nD oD KC LC MC pD"},Q:{"1":"qD"},R:{"1":"rD"},S:{"1":"sD tD"}},B:7,C:"Document.execCommand()",D:true};
