module.exports={A:{A:{"2":"K D E F oC","129":"A B"},B:{"1":"0 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB","129":"C L","1025":"M G N O P"},C:{"2":"1 2 3 4 5 6 7 8 pC NC J RB K D E F A B C L M G N O P SB TB sC tC","513":"0 9 UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC qC rC"},D:{"1":"0 1 2 3 4 5 6 7 8 9 K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC","2":"J RB"},E:{"1":"RB K D E F A B C L M G vC wC xC yC UC HC IC zC 0C 1C VC WC JC 2C KC XC YC ZC aC bC 3C LC cC dC eC fC gC 4C MC hC iC jC kC lC 5C","2":"J uC TC"},F:{"1":"0 1 2 3 4 5 6 7 8 F B C G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z 6C 7C 8C 9C HC mC AD IC"},G:{"388":"E TC BD nC CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UD VC WC JC VD KC XC YC ZC aC bC WD LC cC dC eC fC gC XD MC hC iC jC kC lC"},H:{"2":"YD"},I:{"2":"NC ZD aD bD","388":"J I cD nC dD eD"},J:{"2":"D","388":"A"},K:{"1":"A B C HC mC IC","388":"H"},L:{"388":"I"},M:{"641":"GC"},N:{"388":"A B"},O:{"388":"JC"},P:{"388":"1 2 3 4 5 6 7 8 J fD gD hD iD jD UC kD lD mD nD oD KC LC MC pD"},Q:{"388":"qD"},R:{"388":"rD"},S:{"513":"sD tD"}},B:1,C:"Number input type",D:true};
