module.exports={A:{A:{"2":"K D E oC","132":"F A B"},B:{"1":"0 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB","1028":"C L M G N O P"},C:{"1":"0 9 zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC qC rC","2":"pC NC","260":"J RB K D E F A B C L M G sC tC","1028":"1 2 3 4 5 6 7 8 N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC"},D:{"1":"0 9 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC","548":"1 2 3 4 5 6 7 8 J RB K D E F A B C L M G N O P SB TB","1028":"UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B"},E:{"1":"KC XC YC ZC aC bC 3C LC cC dC eC fC gC 4C MC hC iC jC kC lC 5C","2":"uC TC","548":"J RB K D E F A B C L M G vC wC xC yC UC HC IC zC 0C 1C VC WC JC 2C"},F:{"1":"0 uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z IC","2":"F","548":"B C 6C 7C 8C 9C HC mC AD","1028":"1 2 3 4 5 6 7 8 G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB"},G:{"1":"KC XC YC ZC aC bC WD LC cC dC eC fC gC XD MC hC iC jC kC lC","16":"TC","548":"E BD nC CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UD VC WC JC VD"},H:{"132":"YD"},I:{"1":"I","16":"ZD aD","548":"NC J bD cD nC","1028":"dD eD"},J:{"548":"D A"},K:{"1":"H IC","548":"A B C HC mC"},L:{"1":"I"},M:{"1":"GC"},N:{"132":"A B"},O:{"1":"JC"},P:{"1":"1 2 3 4 5 6 7 8 UC kD lD mD nD oD KC LC MC pD","1028":"J fD gD hD iD jD"},Q:{"1":"qD"},R:{"1":"rD"},S:{"1":"sD tD"}},B:4,C:"Media Queries: resolution feature",D:true};
