'use strict';
require('../../modules/esnext.symbol.custom-matcher');
var WrappedWellKnownSymbolModule = require('../../internals/well-known-symbol-wrapped');

module.exports = WrappedWellKnownSymbolModule.f('customMatcher');
