module.exports={A:{A:{"1":"A B","2":"F oC","8":"K D E"},B:{"1":"C L M G N O P Q H R S T","2":"0 9 U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB"},C:{"1":"1 2 3 4 5 6 7 8 J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S sC tC","2":"0 9 T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC qC rC","4":"NC","8":"pC"},D:{"1":"1 2 3 4 5 6 7 8 J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R S T","2":"0 9 U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC"},E:{"1":"J RB K D E F A B C L M vC wC xC yC UC HC IC zC 0C","2":"G 1C VC WC JC 2C KC XC YC ZC aC bC 3C LC cC dC eC fC gC 4C MC hC iC jC kC lC 5C","8":"uC TC"},F:{"1":"1 2 3 4 5 6 7 8 B C G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B 9C HC mC AD IC","2":"0 F AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z 6C","8":"7C 8C"},G:{"1":"E TC BD nC CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD","2":"UD VC WC JC VD KC XC YC ZC aC bC WD LC cC dC eC fC gC XD MC hC iC jC kC lC"},H:{"2":"YD"},I:{"1":"NC J ZD aD bD cD nC dD eD","2":"I"},J:{"1":"D A"},K:{"1":"B C HC mC IC","2":"A H"},L:{"2":"I"},M:{"2":"GC"},N:{"1":"A B"},O:{"1":"JC"},P:{"1":"1 2 3 4 5 6 7 8 J fD gD hD iD jD UC kD lD mD nD oD KC LC MC pD"},Q:{"1":"qD"},R:{"2":"rD"},S:{"1":"sD","2":"tD"}},B:7,C:"Offline web applications",D:true};
