module.exports={A:{A:{"1":"B","2":"K D E F A oC"},B:{"2":"0 9 C L M G N O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB"},C:{"1":"1 2 3 4 5 6 7 8 L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB","2":"0 9 pC NC J RB K D E F A B C qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC qC rC sC tC"},D:{"1":"1 2 3 4 5 6 7 8 J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB","2":"0 9 qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC"},E:{"1":"E F A B C yC UC HC","2":"J RB K D uC TC vC wC xC","129":"L M G IC zC 0C 1C VC WC JC 2C KC XC YC ZC aC bC 3C LC cC dC eC fC gC 4C MC hC iC jC kC lC 5C"},F:{"1":"1 2 3 4 5 6 7 8 G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB hB jB IC","2":"0 F B C fB gB iB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z 6C 7C 8C 9C HC mC AD"},G:{"1":"E FD GD HD ID JD KD LD MD","2":"TC BD nC CD DD ED","257":"ND OD PD QD RD SD TD UD VC WC JC VD KC XC YC ZC aC bC WD LC cC dC eC fC gC XD MC hC iC jC kC lC"},H:{"2":"YD"},I:{"1":"NC J cD nC dD eD","2":"I ZD aD bD"},J:{"2":"D A"},K:{"1":"IC","2":"A B C H HC mC"},L:{"2":"I"},M:{"2":"GC"},N:{"1":"B","2":"A"},O:{"2":"JC"},P:{"1":"J","2":"1 2 3 4 5 6 7 8 fD gD hD iD jD UC kD lD mD nD oD KC LC MC pD"},Q:{"2":"qD"},R:{"2":"rD"},S:{"1":"sD","2":"tD"}},B:7,C:"SPDY protocol",D:true};
