module.exports={A:{A:{"1":"A B","2":"K D E F oC"},B:{"1":"C L M G N O P","516":"0 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB"},C:{"132":"rB sB tB uB vB wB xB OC yB PC zB 0B 1B","164":"1 2 3 4 5 6 7 8 pC NC J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB sC tC","516":"2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a","1028":"0 9 b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC qC rC"},D:{"420":"1 2 3 4 5 6 7 8 J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB","516":"0 9 pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC"},E:{"1":"A B C L M G UC HC IC zC 0C 1C VC WC JC 2C KC XC YC ZC aC bC 3C LC cC dC eC fC gC 4C MC hC iC jC kC lC 5C","132":"F yC","164":"D E xC","420":"J RB K uC TC vC wC"},F:{"1":"C HC mC AD IC","2":"F B 6C 7C 8C 9C","420":"1 2 3 4 5 6 7 8 G N O P SB TB UB VB WB XB YB ZB aB bB","516":"0 cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z"},G:{"1":"ID JD KD LD MD ND OD PD QD RD SD TD UD VC WC JC VD KC XC YC ZC aC bC WD LC cC dC eC fC gC XD MC hC iC jC kC lC","132":"GD HD","164":"E ED FD","420":"TC BD nC CD DD"},H:{"1":"YD"},I:{"420":"NC J ZD aD bD cD nC dD eD","516":"I"},J:{"420":"D A"},K:{"1":"C HC mC IC","2":"A B","516":"H"},L:{"516":"I"},M:{"1028":"GC"},N:{"1":"A B"},O:{"516":"JC"},P:{"420":"J","516":"1 2 3 4 5 6 7 8 fD gD hD iD jD UC kD lD mD nD oD KC LC MC pD"},Q:{"516":"qD"},R:{"516":"rD"},S:{"164":"sD tD"}},B:4,C:"CSS3 Multiple column layout",D:true};
