"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const configs_1 = __importDefault(require("./configs"));
const rules_1 = __importDefault(require("./rules"));
module.exports = {
    configs: configs_1.default,
    rules: rules_1.default,
};
