"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.importDefault = void 0;
const interopRequireDefault = (obj) => (obj === null || obj === void 0 ? void 0 : obj.__esModule) ? obj : { default: obj };
const importDefault = (moduleName) => interopRequireDefault(require(moduleName)).default;
exports.importDefault = importDefault;
