export = pluginCreator;
/**
 * @type {import('postcss').PluginCreator<Record<string, boolean>>}
 * @param {Record<string, boolean>} config
 * @return {import('postcss').Plugin}
 */
declare function pluginCreator(config?: Record<string, boolean>): import('postcss').Plugin;
declare namespace pluginCreator {
    const postcss: true;
}
