module.exports={A:{A:{"2":"F A B oC","8":"K D E"},B:{"2":"C L M G N O P","8":"Q H R S T U V W X Y Z a b c d e f","584":"g h i j k l m n o p q r","1025":"0 9 s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC qC rC","129":"pC NC sC tC"},D:{"1":"5","8":"1 2 3 4 6 7 8 J RB K D E F A B C L M G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB OC yB PC zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R S T U V W X Y Z a b c d e f","584":"g h i j k l m n o p q r","1025":"0 9 s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QB GC RC SC"},E:{"1":"A B C L M G UC HC IC zC 0C 1C VC WC JC 2C KC XC YC ZC aC bC 3C LC cC dC eC fC gC 4C MC hC iC jC kC lC 5C","260":"J RB K D E F uC TC vC wC xC yC"},F:{"2":"F","8":"1 2 3 4 5 6 7 8 G N O P SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC Q H R QC","584":"S T U V W X Y Z a b c d","1025":"0 e f g h i j k l m n o p q r s t u v w x y z","2052":"B C 6C 7C 8C 9C HC mC AD IC"},G:{"1":"E CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UD VC WC JC VD KC XC YC ZC aC bC WD LC cC dC eC fC gC XD MC hC iC jC kC lC","8":"TC BD nC"},H:{"8":"YD"},I:{"8":"NC J ZD aD bD cD nC dD eD","1025":"I"},J:{"1":"A","8":"D"},K:{"8":"A B C HC mC IC","1025":"H"},L:{"1025":"I"},M:{"1":"GC"},N:{"2":"A B"},O:{"8":"JC"},P:{"1":"2 3 4 5 6 7 8","8":"1 J fD gD hD iD jD UC kD lD mD nD oD KC LC MC pD"},Q:{"8":"qD"},R:{"8":"rD"},S:{"1":"sD tD"}},B:2,C:"MathML",D:true};
